package org.openehealth.ipf.iheclient;

import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.camel.component.hl7.HL7MLLPCodec;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class IHEWebServiceClientConfiguration {

	@Bean
	public org.apache.camel.component.hl7.HL7MLLPCodec hl7codec() {
		HL7MLLPCodec ret = new HL7MLLPCodec();
		ret.setCharset(Charset.defaultCharset());
		return ret;
	}

	/**
	 * Need to load certificates dynamically hence creating sslContext from "raw" certificates rather than the key store
	 * @return
	 * @throws Exception
	 */
	@Bean
	SSLContext sslContext() throws Exception {
		System.err.println("sslcontext");
		Certificate myCert = CertificateFactory.getInstance("X509")
				.generateCertificate(getClass().getResourceAsStream("/partner2.pem"));

		KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
		keyStore.load(null);
		keyStore.setCertificateEntry("partner2", myCert); 

		TrustManagerFactory trustManagerFactory = TrustManagerFactory
				.getInstance(TrustManagerFactory.getDefaultAlgorithm());
		trustManagerFactory.init(keyStore);

		TrustManager trustManagers[] = trustManagerFactory.getTrustManagers();

		for (TrustManager trustManager : trustManagers) {
			if (trustManager instanceof X509TrustManager) {
				SSLContext sslContext = SSLContext.getInstance("TLS");
				sslContext.init(null, new X509TrustManager[] { (X509TrustManager) trustManager }, null);

				return sslContext;
			}
		}
		throw new KeyManagementException("failed to create SSLContext");
	}


}
