/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.openehealth.ipf.iheclient;

import static org.junit.Assert.assertNotNull;

import java.util.Arrays;

import javax.net.ssl.SSLContext;

import org.apache.camel.CamelContext;
import org.apache.camel.component.hl7.HL7MLLPCodec;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Hl7v2Based;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.SpringApplicationConfiguration;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import ca.uhn.hl7v2.model.v231.message.ADT_A01;
import ca.uhn.hl7v2.model.v231.segment.PID;


@RunWith(SpringJUnit4ClassRunner.class)
@EnableAutoConfiguration
@ComponentScan(basePackages = { "org.openehealth.ipf" })
@Configuration
@SpringApplicationConfiguration(classes = {IHEWebServiceClientTest.class, IHEWebServiceClientConfiguration.class})
public class IHEWebServiceClientTest {
	
	@Autowired
	private IHEWebServiceClient iheClient;

	@Autowired
	private ApplicationContext applicationContext;
	
	@Autowired
	private CamelContext camelContext;

	@Autowired
	private HL7MLLPCodec hl7codec;
	
	@Autowired
	private SSLContext context;
	
	@Test
	public void testSmoke() throws Exception {
		assertNotNull(iheClient.getCamelContext());
	}

	@Test
	public void testIti18() throws Exception {
		FindDocumentsQuery qry = new FindDocumentsQuery();
		AssigningAuthority assigningAuthority = new AssigningAuthority("2.16.840.1.113883.3.3388.1.1.2.1.3.10007.1.334737134.1185053597.3189283242.2310779401");
		Identifiable patientId = new Identifiable("f2589bf1-5fa6-4bdf-ac5b-959e2a7752de", assigningAuthority);
		qry.setPatientId(patientId);
		
		qry.setStatus(Arrays.asList(AvailabilityStatus.APPROVED));
		
		QueryResponse result = iheClient.iti18StoredQuery(qry, "parner2.host.name", 443,
				"hl/iheservices/registry?secure=true");
		System.out.println(result);
	}
    
    
	@Test
	public void testIti8() throws Exception {
		ADT_A01 adt=new ADT_A01();
		  adt.initQuickstart("ADT","A01","P");
		  PID pid=adt.getPID();
		  pid.getPatientName(0).getFamilyLastName().getFamilyName().setValue("Doe");
		  pid.getPatientName(0).getGivenName().setValue("John");

		  String result = iheClient.iti8PatientIdentityFeed(adt.encode(), "parner2.host.name", 11000,
				"?secure=true&codec=#hl7codec&sslContext=#sslContext&" +
                "sslProtocols=SSLv3,TLSv1&" +
                "sslCiphers=SSL_RSA_WITH_NULL_SHA,TLS_RSA_WITH_AES_128_CBC_SHA", false);
		System.out.println(result);
	}

}
